;
; !!! Modifications for Noritake CU40026SCPB-T20A (first try...) !!!
;
; MIOS Custom Display Driver for parallel character VFDs
;
; NOTE:	this driver is written for a Futaba M402SD06G display, but with
;       some modifications should also work with other non HD44780
;       compatible parallel displays (for some hints: see below)
;
;       Note also that this driver only supports a single VFD
;
; ==========================================================================
;
; Copyright (C) 2003   Thorsten Klose (tk@midibox.org)
; Copyright (C) 2006   Thomas Stoeckl (seppoman@seppoman.de)
; 
; Licensed for personal non-commercial use only.
; All other rights reserved.
;
; ==========================================================================
;
;       connections:
;
;       core module ->   display
;
;       D0..D7      ->   D0..D7
;       Enable      ->   SEL#
;       R/W         ->   WR#
;       R/S         ->   BUSY (plus a 100 Ohm resistor in series !!!)
;       N/C or +5V  ->   TEST#
;
; ==========================================================================
;
; To activate the driver, you need to select display type 7.
; In MBSID this is done in the main.asm file. Search for the
; USER_Init section (about 2/3 down) and insert the following
; code there:
;
;   ;; select LCD type #7 (custom LCD driver)
;   clrf    MIOS_PARAMETER1    ; variable enable pin not provided
;   clrf    MIOS_PARAMETER2    ; variable enable pin not provided
;   movlw    0x07
;   call    MIOS_LCD_TypeSet
;
; ==========================================================================
;
; Adaptions for other displays:
;
; If you have a different VFD with parallel interface, you'll
; need to find out what the control codes of your display are.
; They are to be entered in the various routines:
;
; movlw 0xXX
; call USER_LCD_Data
;
; sends one Byte to the display, so if 0x10 is the code for
; "place cursor", youll have to send 0x10 and thereafter the
; cursor pos. The Futaba counts the cursor pos continuously,
; so 0x29 (41 decimal) is the first character of the second
; line. If your VFD does this another way you have to change
; the code.
;
; The Futaba M402SD06G doesnt support user-defined characters,
; so theres a table at the end of this driver where the special
; chars can be assigned to suitable ASCII chars of the display.
;
; ==========================================================================

;; Pins of LC-Display
USER_LCD_LAT_D		EQU	LATB	; Pin B.7-0
USER_LCD_PORT_D	EQU     PORTB
USER_LCD_TRIS_D	EQU     TRISB
                	
USER_LCD_LAT_WR		EQU     LATD
USER_LCD_PIN_WR		EQU     6               ; Pin D.6
USER_LCD_LAT_BUSY	EQU     LATD
USER_LCD_PIN_BUSY	EQU     5               ; Pin D.5
USER_LCD_LAT_SEL	EQU     LATD
USER_LCD_PIN_SEL	EQU     7               ; Pin D.7

;; new names for CLCD registers
USER_LCD_STATUS		EQU	MIOS_GLCD_TMP1

#define USER_LCD_STATUS_LCD0_DISABLED	0	; bit0:	if set, first LCD disabled
#define USER_LCD_STATUS_LCD1_DISABLED	1	; bit1:	if set, second LCD disabled -- not provided by this driver!
#define USER_LCD_STATUS_CUR_DISABLED	2	; bit2:	if set, currently selected LCD disabled
#define USER_LCD_STATUS_CUR_LCD		3	; bit3: if cleared: current LCD is first LCD, else second LCD

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when the custom LCD should be initialized
;;  In:  MIOS_LCD_OPTION1 - contains the first LCD option given by MIOS_LCD_TypeSet
;;       MIOS_LCD_OPTION2 - contains the second LCD option given by MIOS_LCD_TypeSet
;;  Out: -
;; --------------------------------------------------------------------------
USER_LCD_Init
	;; notify that no graphical LCD is connected
	bcf	MIOS_BOX_CFG0, MIOS_BOX_CFG0_USE_GLCD

	;; (Initialization of Ports: done in Init_Ports)
	SET_BSR	MIOS_LCD_TIMEOUT1
	clrf	USER_LCD_STATUS, BANKED

	;; 100 ms delay	
	movlw	100
	call	MIOS_Delay

	;; Setup pins
	bsf     USER_LCD_LAT_WR, USER_LCD_PIN_WR
	bsf	USER_LCD_LAT_SEL, USER_LCD_PIN_SEL
	bcf     USER_LCD_LAT_BUSY, USER_LCD_PIN_BUSY
	bsf	TRISD, USER_LCD_PIN_BUSY		; BUSY pin is TriState/input
	
	; initialize LCD
	movlw	0x1B					; ESC
	call	USER_LCD_Data                                            
	movlw	0x49					; Reset Display
	call	USER_LCD_Data                                            
	movlw	0x11					; Normal Display Mode
	call	USER_LCD_Data
	movlw	0x16					; Cursor off
	call	USER_LCD_Data

	; everything ok, make sure that LCD_TIMEOUT, bit 7 is cleared
	bcf	MIOS_LCD_TIMEOUT1, 7, BANKED
	return

;; --------------------------------------------------------------------------
;;  Futaba doesnt use separate Data and Command modes, so just do nothing if
;;  a program should directly access this function
;;  In:  -
;;  Out: -
;; --------------------------------------------------------------------------
USER_LCD_Cmd
        return

;; --------------------------------------------------------------------------
;;  Transfer data to the display
;;  In:  WREG - data byte
;;  Out: -
;; --------------------------------------------------------------------------
USER_LCD_Data
	;; wait until display unbusy
	rcall	USER_LCD_WaitUnbusy

	;; exit if current LCD not available due to timeout
	BIFSET	USER_LCD_STATUS, USER_LCD_STATUS_CUR_DISABLED, BANKED, return

	;; select display
	bcf	USER_LCD_LAT_SEL, USER_LCD_PIN_SEL

	;; store byte in data latch
	movwf	USER_LCD_LAT_D
	nop				;;100 ns
	nop
	nop
	nop
	;; write pulse
	bcf     USER_LCD_LAT_WR, USER_LCD_PIN_WR
	nop				;;175 ns
	nop
	nop
	nop
	nop
	nop
	nop
	bsf     USER_LCD_LAT_WR, USER_LCD_PIN_WR
	nop				;;25 ns

	;; unselect display
	bsf	USER_LCD_LAT_SEL, USER_LCD_PIN_SEL

	return

;; --------------------------------------------------------------------------
;;  This function is NOT called by MIOS, but only used by the custom driver
;;  to wait until the LCD is not busy
;;  In:  -
;;  Out: -
;; --------------------------------------------------------------------------
USER_LCD_WaitUnbusy
	;; exit if current LCD not available due to timeout
	BIFSET	USER_LCD_STATUS, USER_LCD_STATUS_CUR_DISABLED, BANKED, return

	clrf	MIOS_LCD_TIMEOUT0, BANKED
	clrf	MIOS_LCD_TIMEOUT1, BANKED

USER_LCD_WaitUnbusy_Loop
	incf	MIOS_LCD_TIMEOUT0, F, BANKED
	skpnz
	incf	MIOS_LCD_TIMEOUT1, F, BANKED
	bz	USER_LCD_WaitUnbusy_Disable	; leave loop when LCD_TIMEOUT = 0xff. Up to now bit 7 is set and the LCD
						; busy routine will never be called again
	IFSET	PORTD, USER_LCD_PIN_BUSY, rgoto USER_LCD_WaitUnbusy_Loop
	return

USER_LCD_WaitUnbusy_Disable
	;; disable currently selected LCD
	btfss	USER_LCD_STATUS, USER_LCD_STATUS_CUR_LCD, BANKED
	bsf	USER_LCD_STATUS, USER_LCD_STATUS_LCD0_DISABLED, BANKED
	btfsc	USER_LCD_STATUS, USER_LCD_STATUS_CUR_LCD, BANKED
	bsf	USER_LCD_STATUS, USER_LCD_STATUS_LCD1_DISABLED, BANKED
	return

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when the custom LCD should be cleared
;;  In:  -
;;  Out: -
;; --------------------------------------------------------------------------
USER_LCD_Clear
	movlw	0x0E
	call	USER_LCD_Data
	return

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when the cursor should be changed
;;  In: MIOS_LCD_CURSOR_POS - character cursor position  (for CLCDs)
;;  Out: -
;; --------------------------------------------------------------------------
USER_LCD_CursorSet
	movlw	0x1B					; ESC
	call	USER_LCD_Data                                            
	movlw	0x48					; Position Cursor
	rcall	USER_LCD_Data

	SET_BSR MIOS_LCD_CURSOR_POS
	movf MIOS_LCD_CURSOR_POS, W, BANKED

	; here you'll have to experiment!
	BIFSET MIOS_LCD_CURSOR_POS, 6, BANKED, addlw -(0x40-0x28) 

	rgoto USER_LCD_Data

;; --------------------------------------------------------------------------
;;  This function is called by MIOS when a character should be printed
;;  In:  WREG - character
;;  Out: -
;; --------------------------------------------------------------------------
USER_LCD_PrintChar
        ;;check if W in 0x00..0x07, i.e. special chars
        movwf	MIOS_GLCD_TMP2, BANKED
        movlw	0x08
        movwf	MIOS_GLCD_TMP3, BANKED
        movf	MIOS_GLCD_TMP2, W, BANKED

	;; if no special char, continue
        cpfsgt	MIOS_GLCD_TMP3, BANKED
	rgoto	USER_LCD_PrintChar_Cont

	;; read replacement char from table
	addlw	USER_LCD_SPECIAL_TABLE & 0xff
	movwf	TBLPTRL
	clrf	TBLPTRH
	movlw	USER_LCD_SPECIAL_TABLE >> 8
	addwfc	TBLPTRH

	tblrd*+
	movf	TABLAT, W

USER_LCD_PrintChar_Cont
        ;; Print Character
	rgoto	USER_LCD_Data

;; --------------------------------------------------------------------------
;;  FUNCTION: USER_LCD_SpecialCharInit
;;  NOTE: No definable special characters...
;; --------------------------------------------------------------------------
USER_LCD_SpecialCharInit
	return

;; --------------------------------------------------------------------------
;; replacement table for special chars - enter values for suitable ASCII chars
;; (depending on the character set of the display)
;; --------------------------------------------------------------------------
USER_LCD_SPECIAL_TABLE		; For MBSID:
	db	0x3c, 0x3e	; Left Arrow, Right Arrow
	db	0x2a, 0x2a	; Inverted 1, Inverted 2
	db	0x2a, 0x2a	; Inverted 3, Inverted 4
	db	0x2a, 0x3d	; Inverted -, L/R Arrow
